"use strict";
const android_livesync_service_1 = require("../../../mobile/android/android-livesync-service");
const path = require("path");
const helpers = require("../../../helpers");
class AppBuilderAndroidLiveSyncService extends android_livesync_service_1.AndroidLiveSyncService {
    constructor(_device, $fs, $mobileHelper, $options) {
        super(_device, $fs, $mobileHelper);
        this._device = _device;
        this.$options = $options;
    }
    refreshApplication(deviceAppData) {
        return (() => {
            let commands = [this.liveSyncCommands.SyncFilesCommand()];
            if (this.$options.watch || this.$options.file) {
                commands.push(this.liveSyncCommands.RefreshCurrentViewCommand());
            }
            else {
                commands.push(this.liveSyncCommands.ReloadStartViewCommand());
            }
            this.livesync(deviceAppData.appIdentifier, deviceAppData.deviceProjectRootPath, commands).wait();
        }).future()();
    }
    removeFiles(appIdentifier, localToDevicePaths) {
        return (() => {
            if (localToDevicePaths && localToDevicePaths.length) {
                let deviceProjectRootPath = localToDevicePaths[0].deviceProjectRootPath;
                let commands = _.map(localToDevicePaths, ldp => {
                    let relativePath = path.relative(deviceProjectRootPath, ldp.getDevicePath()), unixPath = helpers.fromWindowsRelativePathToUnix(relativePath);
                    return this.liveSyncCommands.DeleteFile(unixPath);
                });
                this.livesync(appIdentifier, deviceProjectRootPath, commands).wait();
            }
        }).future()();
    }
}
exports.AppBuilderAndroidLiveSyncService = AppBuilderAndroidLiveSyncService;
$injector.register("androidLiveSyncServiceLocator", { factory: AppBuilderAndroidLiveSyncService });
