"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
const fs = require("fs");
const Future = require("fibers/future");
const path = require("path");
const minimatch = require("minimatch");
const decorators = require("./decorators");
const injector = require("./yok");
const crypto = require("crypto");
const shelljs = require("shelljs");
let FileSystem_1 = class FileSystem {
    constructor($injector) {
        this.$injector = $injector;
    }
    zipFiles(zipFile, files, zipPathCallback) {
        let $logger = this.$injector.resolve("logger");
        let zipstream = require("zipstream");
        let zip = zipstream.createZip({ level: 9 });
        let outFile = fs.createWriteStream(zipFile);
        zip.pipe(outFile);
        let result = new Future();
        outFile.on("error", (err) => result.throw(err));
        let fileIdx = -1;
        let zipCallback = () => {
            fileIdx++;
            if (fileIdx < files.length) {
                let file = files[fileIdx];
                let relativePath = zipPathCallback(file);
                relativePath = relativePath.replace(/\\/g, "/");
                $logger.trace("zipping as '%s' file '%s'", relativePath, file);
                zip.addFile(fs.createReadStream(file), { name: relativePath }, zipCallback);
            }
            else {
                outFile.on("finish", () => result.return());
                zip.finalize((bytesWritten) => {
                    $logger.debug("zipstream: %d bytes written", bytesWritten);
                    outFile.end();
                });
            }
        };
        zipCallback();
        return result;
    }
    unzip(zipFile, destinationDir, options, fileFilters) {
        return (() => {
            let shouldOverwriteFiles = !(options && options.overwriteExisitingFiles === false);
            let isCaseSensitive = !(options && options.caseSensitive === false);
            let $hostInfo = this.$injector.resolve("$hostInfo");
            this.createDirectory(destinationDir).wait();
            let proc;
            if ($hostInfo.isWindows) {
                proc = path.join(__dirname, "resources/platform-tools/unzip/win32/unzip");
            }
            else if ($hostInfo.isDarwin) {
                proc = "unzip";
            }
            else if ($hostInfo.isLinux) {
                proc = "unzip";
            }
            if (!isCaseSensitive) {
                zipFile = this.findFileCaseInsensitive(zipFile);
            }
            let args = _.flatten(["-b",
                shouldOverwriteFiles ? "-o" : "-n",
                isCaseSensitive ? [] : "-C",
                zipFile,
                fileFilters || [],
                "-d",
                destinationDir]);
            let $childProcess = this.$injector.resolve("childProcess");
            $childProcess.spawnFromEvent(proc, args, "close", { stdio: "ignore", detached: true }).wait();
        }).future()();
    }
    findFileCaseInsensitive(file) {
        let dir = path.dirname(file);
        let basename = path.basename(file);
        let entries = this.readDirectory(dir).wait();
        let match = minimatch.match(entries, basename, { nocase: true, nonegate: true, nonull: true })[0];
        let result = path.join(dir, match);
        return result;
    }
    exists(path) {
        let future = new Future();
        fs.exists(path, (exists) => future.return(exists));
        return future;
    }
    tryExecuteFileOperation(path, operation, enoentErrorMessage) {
        return (() => {
            try {
                operation().wait();
            }
            catch (e) {
                this.$injector.resolve("$logger").trace("tryExecuteFileOperation failed with error %s.", e);
                if (enoentErrorMessage) {
                    let message = (e.code === "ENOENT") ? enoentErrorMessage : e.message;
                    this.$injector.resolve("$errors").failWithoutHelp(message);
                }
            }
        }).future()();
    }
    deleteFile(path) {
        let future = new Future();
        fs.unlink(path, (err) => {
            if (err && err.code !== "ENOENT") {
                future.throw(err);
            }
            else {
                future.return();
            }
        });
        return future;
    }
    deleteDirectory(directory) {
        let future = new Future();
        shelljs.rm("-rf", directory);
        let err = shelljs.error();
        if (err !== null) {
            future.throw(new Error(err));
        }
        else {
            future.return();
        }
        return future;
    }
    getFileSize(path) {
        return (() => {
            let stat = this.getFsStats(path).wait();
            return stat.size;
        }).future()();
    }
    futureFromEvent(eventEmitter, event) {
        let future = new Future();
        eventEmitter.once(event, function () {
            let args = _.toArray(arguments);
            if (event === "error") {
                let err = args[0];
                future.throw(err);
                return;
            }
            switch (args.length) {
                case 0:
                    future.return();
                    break;
                case 1:
                    future.return(args[0]);
                    break;
                default:
                    future.return(args);
                    break;
            }
        });
        return future;
    }
    createDirectory(path) {
        let future = new Future();
        require("mkdirp")(path, (err) => {
            if (err) {
                future.throw(err);
            }
            else {
                future.return();
            }
        });
        return future;
    }
    readDirectory(path) {
        let future = new Future();
        fs.readdir(path, (err, files) => {
            if (err) {
                future.throw(err);
            }
            else {
                future.return(files);
            }
        });
        return future;
    }
    readFile(filename) {
        let future = new Future();
        fs.readFile(filename, (err, data) => {
            if (err) {
                future.throw(err);
            }
            else {
                future.return(data);
            }
        });
        return future;
    }
    readText(filename, options) {
        options = options || { encoding: "utf8" };
        if (_.isString(options)) {
            options = { encoding: options };
        }
        if (!options.encoding) {
            options.encoding = "utf8";
        }
        let future = new Future();
        fs.readFile(filename, options, (err, data) => {
            if (err) {
                future.throw(err);
            }
            else {
                future.return(data);
            }
        });
        return future;
    }
    readJson(filename, encoding) {
        return (() => {
            let data = this.readText(filename, encoding).wait();
            if (data) {
                return JSON.parse(data.replace(/^\uFEFF/, ""));
            }
            return null;
        }).future()();
    }
    writeFile(filename, data, encoding) {
        return (() => {
            this.createDirectory(path.dirname(filename)).wait();
            let future = new Future();
            fs.writeFile(filename, data, { encoding: encoding }, (err) => {
                if (err) {
                    future.throw(err);
                }
                else {
                    future.return();
                }
            });
            future.wait();
        }).future()();
    }
    appendFile(filename, data, encoding) {
        let future = new Future();
        fs.appendFile(filename, data, { encoding: encoding }, (err) => {
            if (err) {
                future.throw(err);
            }
            else {
                future.return();
            }
        });
        return future;
    }
    writeJson(filename, data, space, encoding) {
        if (!space) {
            space = this.getIndentationCharacter(filename).wait();
        }
        return this.writeFile(filename, JSON.stringify(data, null, space), encoding);
    }
    copyFile(sourceFileName, destinationFileName) {
        if (path.resolve(sourceFileName) === path.resolve(destinationFileName)) {
            return Future.fromResult();
        }
        let res = new Future();
        this.createDirectory(path.dirname(destinationFileName)).wait();
        let source = this.createReadStream(sourceFileName);
        let target = this.createWriteStream(destinationFileName);
        source.on("error", (e) => {
            if (!res.isResolved()) {
                res.throw(e);
            }
        });
        target.on("finish", () => {
            if (!res.isResolved()) {
                res.return();
            }
        })
            .on("error", (e) => {
            if (!res.isResolved()) {
                res.throw(e);
            }
        });
        source.pipe(target);
        return res;
    }
    createReadStream(path, options) {
        return fs.createReadStream(path, options);
    }
    createWriteStream(path, options) {
        return fs.createWriteStream(path, options);
    }
    chmod(path, mode) {
        let future = new Future();
        fs.chmod(path, mode, (err) => {
            if (err) {
                future.throw(err);
            }
            else {
                future.return();
            }
        });
        return future;
    }
    getFsStats(path) {
        let future = new Future();
        fs.stat(path, (err, data) => {
            if (err) {
                future.throw(err);
            }
            else {
                future.return(data);
            }
        });
        return future;
    }
    getLsStats(path) {
        let future = new Future();
        fs.lstat(path, (err, data) => {
            if (err) {
                future.throw(err);
            }
            else {
                future.return(data);
            }
        });
        return future;
    }
    getUniqueFileName(baseName) {
        return (() => {
            if (!this.exists(baseName).wait()) {
                return baseName;
            }
            let extension = path.extname(baseName);
            let prefix = path.basename(baseName, extension);
            for (let i = 2;; ++i) {
                let numberedName = prefix + i + extension;
                if (!this.exists(numberedName).wait()) {
                    return numberedName;
                }
            }
        }).future()();
    }
    isEmptyDir(directoryPath) {
        return (() => {
            let directoryContent = this.readDirectory(directoryPath).wait();
            return directoryContent.length === 0;
        }).future()();
    }
    isRelativePath(p) {
        let normal = path.normalize(p);
        let absolute = path.resolve(p);
        return normal !== absolute;
    }
    ensureDirectoryExists(directoryPath) {
        return (() => {
            if (!this.exists(directoryPath).wait()) {
                this.createDirectory(directoryPath).wait();
            }
        }).future()();
    }
    rename(oldPath, newPath) {
        let future = new Future();
        fs.rename(oldPath, newPath, (err) => {
            if (err) {
                future.throw(err);
            }
            else {
                future.return();
            }
        });
        return future;
    }
    renameIfExists(oldPath, newPath) {
        return (() => {
            try {
                this.rename(oldPath, newPath).wait();
                return true;
            }
            catch (e) {
                if (e.code === "ENOENT") {
                    return false;
                }
                throw e;
            }
        }).future()();
    }
    symlink(sourcePath, destinationPath, type) {
        let future = new Future();
        fs.symlink(sourcePath, destinationPath, type, (err) => {
            if (err) {
                future.throw(err);
            }
            else {
                future.return();
            }
        });
        return future;
    }
    closeStream(stream) {
        let future = new Future();
        stream.close((err, data) => {
            if (err) {
                future.throw(err);
            }
            else {
                future.return();
            }
        });
        return future;
    }
    setCurrentUserAsOwner(path, owner) {
        return (() => {
            let $childProcess = this.$injector.resolve("childProcess");
            if (!this.$injector.resolve("$hostInfo").isWindows) {
                let chown = $childProcess.spawn("chown", ["-R", owner, path], { stdio: "ignore", detached: true });
                this.futureFromEvent(chown, "close").wait();
            }
        }).future()();
    }
    enumerateFilesInDirectorySync(directoryPath, filterCallback, opts, foundFiles) {
        foundFiles = foundFiles || [];
        if (!this.exists(directoryPath).wait()) {
            let $logger = this.$injector.resolve("logger");
            $logger.warn('Could not find folder: ' + directoryPath);
            return foundFiles;
        }
        let contents = this.readDirectory(directoryPath).wait();
        for (let i = 0; i < contents.length; ++i) {
            let file = path.join(directoryPath, contents[i]);
            let stat = this.getFsStats(file).wait();
            if (filterCallback && !filterCallback(file, stat)) {
                continue;
            }
            if (stat.isDirectory()) {
                if (opts && opts.enumerateDirectories) {
                    foundFiles.push(file);
                }
                if (opts && opts.includeEmptyDirectories && this.readDirectory(file).wait().length === 0) {
                    foundFiles.push(file);
                }
                this.enumerateFilesInDirectorySync(file, filterCallback, opts, foundFiles);
            }
            else {
                foundFiles.push(file);
            }
        }
        return foundFiles;
    }
    getFileShasum(fileName, options) {
        let future = new Future();
        let algorithm = (options && options.algorithm) || "sha1";
        let encoding = (options && options.encoding) || "hex";
        let logger = this.$injector.resolve("$logger");
        let shasumData = crypto.createHash(algorithm);
        let fileStream = this.createReadStream(fileName);
        fileStream.on("data", (data) => {
            shasumData.update(data);
        });
        fileStream.on("end", () => {
            let shasum = shasumData.digest(encoding);
            logger.trace(`Shasum of file ${fileName} is ${shasum}`);
            future.return(shasum);
        });
        fileStream.on("error", (err) => {
            future.throw(err);
        });
        return future;
    }
    readStdin() {
        let future = new Future();
        let buffer = '';
        process.stdin.on('data', (data) => buffer += data);
        process.stdin.on('end', () => future.return(buffer));
        return future;
    }
    rm(options, ...files) {
        shelljs.rm(options, files);
    }
    deleteEmptyParents(directory) {
        return (() => {
            let parent = this.exists(directory).wait() ? directory : path.dirname(directory);
            while (this.isEmptyDir(parent).wait()) {
                this.deleteDirectory(parent).wait();
                parent = path.dirname(parent);
            }
        }).future()();
    }
    getIndentationCharacter(filePath) {
        return (() => {
            if (!this.exists(filePath).wait()) {
                return FileSystem_1.DEFAULT_INDENTATION_CHARACTER;
            }
            let fileContent = this.readText(filePath).wait().trim();
            let matches = fileContent.match(FileSystem_1.JSON_OBJECT_REGEXP);
            if (!matches || !matches[1]) {
                return FileSystem_1.DEFAULT_INDENTATION_CHARACTER;
            }
            let indentation = matches[1];
            return indentation[0] === " " ? indentation : FileSystem_1.DEFAULT_INDENTATION_CHARACTER;
        }).future()();
    }
};
let FileSystem = FileSystem_1;
FileSystem.DEFAULT_INDENTATION_CHARACTER = "\t";
FileSystem.JSON_OBJECT_REGEXP = new RegExp(`{\\r*\\n*(\\W*)"`, "m");
__decorate([
    decorators.exportedPromise("fs")
], FileSystem.prototype, "getFileSize", null);
FileSystem = FileSystem_1 = __decorate([
    injector.register("fs")
], FileSystem);
exports.FileSystem = FileSystem;
